"""
SSH configuration file parser
-----------------------------

Uses paramiko's file parser and provides some convenience functions.

Example::

    import schneider.config


    hostname = "my_host"

    # use default path
    config = schneider.config.parse()

    # check ifget entry for the defined hostname if defined
    if not hostname in config.get_hostnames():
        print("error: unknown host", hostname)
    else:        
        # get its address
        address = config.lookup(hostname)["hostname"]
        # alternative: schneider.config.get_address(config, hostname)

        # multiple identity files are allowed, so this is a list
        keyfile = config.lookup(hostname)["identityfile"][0]
        # alternative: schneider.config.get_keyfile(config, hostname)

        # now use address and keyfile to create a new agent

"""

from os.path import expanduser, normpath

from paramiko import SSHConfig


class HostNotFound(Exception):
    """Host not found in local SSH config"""

    pass


class NoKeyFile(Exception):
    """Host entry has no keyfile"""

    pass


def parse(path="~/.ssh/config"):
    """
    Parse the given SSH config file; defaults to `~/.ssh/config`.

    :param: path - path to local SSH config file
    :return: SSHConfig instance
    """

    return SSHConfig.from_path(normpath(expanduser(path)))


def get_keyfile(ssh_config, hostname):
    """Get the first keyfile from given host, if any.

    :exception: HostNotFound, NoKeyFile
    :param: ssh_config - SSH object returned by `parse`
    :param: hostname - Name of the config entry
    :return: first keyfile path
    """

    if not hostname in ssh_config.get_hostnames():
        raise HostNotFound(ssh_config)
    else:
        host_config = ssh_config.lookup(hostname)
        if not "identityfile" in host_config:
            raise NoKeyFile(host_config)
        else:
            return host_config["identityfile"][0]


def get_address(ssh_config, hostname):
    """Get address of given host.

    :exception: HostNotFound
    :param: ssh_config - SSH object returned by `parse`
    :param: hostname - Name of the config file entry
    :return: host address
    """

    if not hostname in ssh_config.get_hostnames():
        raise HostNotFound(ssh_config)
    else:
        host_config = ssh_config.lookup(hostname)
        return host_config["hostname"]

