"""schneider manager module
"""

import os

from schneider.agent import SSHAgent


class UnkownAgent(Exception):
    pass


class Manager:
    """Manager for SSHAgent instances.

    Example::

        # create a agent
        agent = schneider.agent.SSHAgent("test-agent")
        agent.create()
        # add keys, other agents ...

        # create a manager
        manager = schneider.manager.Manager()

        # add agent(s) to manager
        manager.add_agent(agent)

        # activate a manager by name
        manager.activate_agent(agent.name)

        # use paramiko, fabric etc.

        # deactivate agent (=restores the previous environment)
        manager.deactive_agent(agent.name)

        # delete agent(s)
        manager.del_agent(agent.name)

    """

    def __init__(self):
        self.orig_ssh_auth_sock = None
        self.orig_ssh_pid = None
        self.agents = {}
        self.active = None

    def __del__(self):
        for agent in self.agents:
            del agent

    def add_agent(self, agent):
        """Add an agent instance.

        :param: agent - agent instance
        """

        if not agent.name in self.agents:
            self.agents[agent.name] = agent

    def del_agent(self, agent_name):
        """Delete agent from manager.

        :param: agent_name - name of agent to delete
        """

        if agent_name in self.agents:
            del self.agents[agent_name]

    def activate_agent(self, agent_name):
        """Activate the environment of the given agent.

        Raises `UnkownAgent` if the given agent name is unknown.

        :param: agent_name - name of agent to use
        """

        if not agent_name in self.agents:
            raise UnkownAgent(agent)
        else:
            if self.active and len(self.active)>0:
                self.deactive_agent(self.active)

            # save orig ssh env vars
            self.orig_ssh_auth_sock = os.environ["SSH_AUTH_SOCK"]
            self.orig_ssh_pid = os.environ["SSH_AGENT_PID"]

            # set custon ssh env vars from agent
            os.environ = os.environ | self.agents[agent_name].env()
            self.active = agent_name

    def deactivate_agent(self, agent_name):
        """Deactive the environment of the given agent.

        Raises `UnkownAgent` if the given agent name is unknown.

        :param: agent_name - name of the agent to use
        """

        if not agent_name in self.agents:
            raise UnkownAgent(agent)
        else:
            if self.active == agent_name:
                # restore orig ssh env vars
                os.environ = os.environ | { "SSH_AGENT_PID": self.orig_ssh_pid, "SSH_AUTH_SOCK": self.orig_ssh_auth_sock }

